@@ -1,11 +1,9 @@
-from __future__ import print_function
 import time
 import tempfile
 import shutil
 import unittest
 import email.utils
 from contextlib import contextmanager
-import pytest
 
 from scrapy.http import Response, HtmlResponse, Request
 from scrapy.spiders import Spider
@@ -84,8 +82,8 @@ class _BaseTest(unittest.TestCase):
 
     def assertEqualRequestButWithCacheValidators(self, request1, request2):
         self.assertEqual(request1.url, request2.url)
-        assert not b'If-None-Match' in request1.headers
-        assert not b'If-Modified-Since' in request1.headers
+        assert b'If-None-Match' not in request1.headers
+        assert b'If-Modified-Since' not in request1.headers
         assert any(h in request2.headers for h in (b'If-None-Match', b'If-Modified-Since'))
         self.assertEqual(request1.body, request2.body)
 
@@ -148,17 +146,13 @@ class FilesystemStorageTest(DefaultStorageTest):
 
     storage_class = 'scrapy.extensions.httpcache.FilesystemCacheStorage'
 
+
 class FilesystemStorageGzipTest(FilesystemStorageTest):
 
     def _get_settings(self, **new_settings):
         new_settings.setdefault('HTTPCACHE_GZIP', True)
         return super(FilesystemStorageTest, self)._get_settings(**new_settings)
 
-class LeveldbStorageTest(DefaultStorageTest):
-
-    pytest.importorskip('leveldb')
-    storage_class = 'scrapy.extensions.httpcache.LeveldbCacheStorage'
-
 
 class DummyPolicyTest(_BaseTest):
 
@@ -507,5 +501,6 @@ class RFC2616PolicyTest(DefaultStorageTest):
                 self.assertEqualResponse(res1, res2)
                 assert 'cached' in res2.flags
 
+
 if __name__ == '__main__':
     unittest.main()
